function [l n comp compl centers] = meanShift(data,params)
% Mean Shift algorithm. Algorithm uses kernel density estimation method to
% estimate underlying pdf distribution. Pdf estimation can be estimated
% based on the leave-one-out kernel width estimation or variable kernel
% width. 
% IN:   data : nxN data matrix(or vector)
%       params : output of fitkde algorithm
% OUT:  l : num of reduced cluster number
%       n : num of total clusters
%       comp: Kx1 structure array containing the components, each element
%            (field "c") is a list of vertices.
%       compl: Nx1 list containing the component label for each vertex.
cVec = 'cmkgcmkgcmkgcmkgcmkgcmkgcmkgcmkgcmkgcmkgcmkgcmkgcmkgcmkgcmkgcmk';%, cVec = [cVec cVec];
[dim,N] = size(data);
if params.ani
    invCxi = params.invCxi;
    invCi = params.invCi;
    D = 0;
    for i=1:N
        D = D + 1/det(invCi(:,(i-1)*3+1:i*3));
    end
    D = D/N; % Avarage of det's : to give a idea of cluster cov
end
% close all,figure(1),clf,hold on,plot(data(1,:),data(2,:),'.')
% figure(2),clf,hold on,plot(data(1,:),data(2,:),'.')
mspoint = zeros(dim,N);
for i=1:N
    if ~mod(i,1000)
        i
    end
    point = data(:,i);
    point_ex = zeros(size(point));
    iter = 0;
    while sum(abs(point-point_ex))>.01 && iter<100
        point_ex = point;
        [K] = kernel_matrix(point,data,dim,1,N,params);
%         if params.ani
%             num = sum(repmat(K,dim,1).* invCxi,2);
%             denum = reshape(sum(reshape(reshape(repmat(K,dim*dim,1),...
%                 dim*dim*N,1).*invCi(:),dim*dim,N),2),dim,dim);
%             invdenum = inv(denum);
%         else
            num = sum(repmat(K,dim,1).*data,2);
            denum = sum(K,2);
            invdenum = 1/denum;
%         end
        msupdate = invdenum*num;
        point = msupdate;
%         figure(1),plot(msupdate(1,:),msupdate(2,:),'g.')
        iter = iter+1;
    end
%     figure(1),plot(msupdate(1,:),msupdate(2,:),'r.')
    mspoint(:,i) = msupdate; 
end
%%
% figure, hist3(mspoint',[90 90])
W = squareform(pdist(mspoint'));
if params.ani
    A = W<D*1;
else
    A = W<params.sptSig*1;
end
[l, comp, compl] = conncomp(A);
if params.visual
    figure(params.fig),hold on,plot3(data(3,:),data(2,:),data(1,:),'.')
end
nn=0;

compll = compl;
n = [];
for i=1:l
    t=ismember(compll,i);
     sum(t)
    if sum(t)>N/300
        nn = nn+1;
        n = union(n,i);
        centers(:,nn) = mean(mspoint(:,t),2);
        if params.visual
        figure(params.fig),plot3(data(3,t),data(2,t),data(1,t),[cVec(nn) '.'])
        hold on,plot3(centers(3,nn),centers(2,nn),centers(1,nn),'rs')
        end
    else
%         compl = setdiff(compl,i);
    end
end
% comp = comp(n);
%%
% [K, comp, compl] = conncomp(N,I,J) or conncomp(A)
% Connected components of a graph
%
% Finds the connected components of an undirected graph. The graph can be
% given by its adjacency matrix A, or by giving its number of vertices N and
% its edges I<->J. It uses a depth-first search which is O(E) on average,
% where E is the number of edges.
%
% In:
%   A: NxN adjacency matrix.
%   N: number of vertices.
%   I, J: Ex1 vectors containing the edges (between I(e) and J(e)).
% Out:
%   K: number of connected components.
%   comp: Kx1 structure array containing the components, each element
%      (field "c") is a list of vertices.
%   compl: Nx1 list containing the component label for each vertex.

% Copyright (c) 2003 by Miguel A. Carreira-Perpinan
end
function [K, comp, compl] = conncomp(N,I,J)

if nargin==1
  A = N; N = size(A,1);
else
  % Build adjacency matrix
  A = sparse(I,J,ones(length(I),1),N,N);
  % Ensure it is symmetric (this actually reduces its storage size, since it
  % becomes a logical sparse array)
  A = (A+A') > 0;
end

L = 1:N;
K = 0;
compl = zeros(N,1);

while ~isempty(L)
  M = L(1);				% Pick point from L
  L = L(2:end);				% L = setdiff(L,M);
  K = K + 1; C = M;
  while ~isempty(M)			% Compute Kth component
    n = L(find(A(L,M(1))));		%#ok<FNDSB> % Neighbours of M(1) in L
    M = union(n,M(2:end));		% M = setdiff(M,M(1)) U n;
    L = setdiff(L,n);
    C = union(C,n);
  end
  comp(K).c = C;
  compl(C) = K;				% Vertex memberships
end
end

% Another way to compute the connected components (far slower in this
% implementation): through the reachability matrix of the graph, R = A^Inf. 
% R can be computed efficiently as logical powers of 2 of A (A^n = R for n <
% log2(N-1)), or recursively by computing the reachable neighbours of a
% given point. The implementation below, though vectorised, can be made more
% efficient with loops and logical operations.

% $$$ % Adjacency matrix
% $$$ A = sparse(I,J,ones(length(I),1),N,N);
% $$$ A = (A+A') > 0;
% $$$ A = full(A);
% $$$ % Force ones along the diagonal
% $$$ A(sub2ind(size(A),1:N,1:N)) = 1;
% $$$ 
% $$$ % 1. Compute reachability matrix R = A^Inf.
% $$$ R = A; oldR = zeros(size(A));
% $$$ while any(xor(oldR(:),R(:)))
% $$$   oldR = R; R = R^2 > 0;
% $$$ end
% $$$ % 2. Compute labels.
% $$$ list = (1:N)'; labels = zeros(N,1); K = 0;
% $$$ while ~all(isnan(list))
% $$$   i = min(list); c = R(:,i);
% $$$   K = K + 1;
% $$$   labels(c) = K;
% $$$   list(c) = NaN;
% $$$ end


