function mainOlfactory(directory,dataset)
% Main of Olfactory Projection Fibers

% Data Information
% Experimental Procedures:
% Dataset owner: GSXE Jefferis, Department of Zoology, University of Cambridge
% Species: Drosophila
% Strain: y w hs-FLP UAS-mCD8-GFP/+ or Y; FRTG13 tubP-GAL80/FRTG13 GAL4-GH146 UAS-mCD8-GFP
% Nervous System Region: Olfactory Bulb
% Fiber type: Axons
% Labeling Method: GFP
% Image Acquisition Method: 2-channel confocal microscopy
% Tracing Method: Neurolucida (Colchester, VT)
% Objective Lens: 40x oil (NA=1.3) with 1.5x zoom
% See Jefferis GS et al., 2007, Cell Vol 128(6) for general information related to the data
% Z distance between successive images within an image stack: 3.03 pixel
% Spacings : X&Y = 0.3296485 microns, Z =1 microns
%%%%%%%%%%%%%%%%%
% Manual Reconstruction Information
% OP_1 (X,Y,Z): (30.979,429.04,0.000); Training tree; Manual reconstruction provided
% OP_2 (X,Y,Z): (0.72501,391.08,25); Training tree; Manual reconstruction provided
% OP_3 (X,Y,Z): (93.742,179,38); Training tree; Manual reconstruction provided
% OP_4 (X,Y,Z): (128.2,504.37,0.303); Scored tree; Manual reconstruction not provided
% OP_5 (X,Y,Z): (185.7,264.02,33); Scored tree; Manual reconstruction not provided
% OP_6 (X,Y,Z): (15.074,412.01,10); Scored tree; Manual reconstruction not provide
%%%%%%%%%%%%%%%%%
% NOTE: x,y,z (above) is defined for image domain, in order to make
% notation consistent with Matlab, I converted the indexces to X,Y,Z, so in
% the code the order is as follows
%               --- Z = x-->
%              /|
%             / |
%       Y = y/  |
%               |
%            X = z
if nargin==0
    directory = 'C:/ERHAN/WORK/DATA/DIADEM/Olfactory Projection Fibers v2/Image Stacks/Qualifier Image Stack New/';
end

addpath(genpath([cd '/mfiles']))
beginn = 1;
if (beginn)
%     clear all;close all;clc
    % Keep parameters inside params
    params.dim.x = 512;
    params.dim.y = 512;
    % # of images:
    % OP1:60;OP2:88;OP3:62;OP4:67;OP5:76;OP6:101;
    
    params.dataset = dataset;
    params.zdist = 1;%3.03;
    params.spacing.xyz = [0.3296485, 0.3296485, 1];
    
    switch params.dataset
        %     convert seed points to matlab indecies: X-Y-Z = z-y-x
        case 1
            seedxyz = [30.979,429.04,0.000];
            seed = [seedxyz(3) seedxyz(2) seedxyz(1)];
            seed = seed + [1 0 0];
        case 2
            seedxyz = [0.72501,391.08,25];
            seed = [seedxyz(3) seedxyz(2) seedxyz(1)];
            seed = seed + [1 0 0];
            params.dim.z = 88;
            tangVecprev = [1;-1;1];   %initial direction of pc
            
            
        case 3
            seedxyz = [93.742,179,38];
            seed = [seedxyz(3) seedxyz(2) seedxyz(1)];
            seed = seed + [1 0 0];
        case 4
            for i=1
                seedxyz = [128.2,504.37,0.303];
                seed = [seedxyz(3) seedxyz(2) seedxyz(1)];
                seed = seed + [1 0 0];
                params.dim.z = 67;
                
                params.tangVecprev = [0;0;-1];   %initial direction of pc
                params.sptSig = 5;
                params.colSig = .5;
                params.kernelWeightFactor = .2; %power of the weight term in tracing step
                params.Factors.neig=2;  % weights of vectorial tracing
                params.Factors.vprev=1;
                params.Tracing.lenTang=1;    %tangent vec
                params.Tracing.lenVal=1;  %color
                params.branchFibStart = 4;
                params.Bif.varThr = false;
                params.Bif.remLowThr = .5;
                params.Bif.Rmin = 4;
                params.Bif.Rmax = 14;
                params.Bif.clusterSize = 3;
                params.Bif.ClusterDistThr = 5;
                params.Bif.centroidImtensityThr = .3;
                params.Bif.intensityThr = .1;
                params.remIntensityThr = .3;
                params.IntensityThr = .2;
                
                params.MS.sptSig = 3;    %to find bifurcations use a smaller spt. kernel
                params.MS.colSig = .5;    %to find bifurcations use a smaller spt. kernel
                params.varSig = true;
                
                % for 17 => 7; sum(cv) = 30 => 2.5;  .45*sum(cv)+14.65
                % for 17 => 5; sum(cv) = 30 => 2.5;  .25*sum(cv)+9.25
                params.varSigMax = 7;
                params.varSigMin = 2.5;
                params.varSigMinBound = 2.5;

                x=[17 27];
                y=[params.varSigMax params.varSigMin];
                a=(y(2)-y(1))/(x(2)-x(1));
                b=y(1)-a*x(1);
                params.varSigCoef = [a,b];
                
                params.smoothImages = false;
                
                params.forePerc = 100;
                params.filt = false;

            end
            
            
        case 5
            for i=1
                seedxyz = [185.7,264.02,33];
                seed = [seedxyz(3) seedxyz(2) seedxyz(1)];
                seed = seed + [1 0 0];
                params.dim.z = 76;
                params.tangVecprev = [0;-1;1];   %initial direction of pc
                params.sptSig = 5;
                params.colSig = .5;
                params.kernelWeightFactor = .2; %power of the weight term in tracing step
                params.Factors.neig=.5;  % weights of vectorial tracing
                params.Factors.vprev=1;
                params.Tracing.lenTang=1;    %tangent vec
                params.Tracing.lenVal=1;  %color
                params.branchFibStart = 4;
                params.Bif.varThr = false;
                params.Bif.remLowThr = .5;
                params.Bif.Rmin = 4;
                params.Bif.Rmax = 14;
                params.Bif.clusterSize = 3;
                params.Bif.ClusterDistThr = 5;
                params.Bif.centroidImtensityThr = .02;
                params.Bif.intensityThr = .01;
                params.remIntensityThr = .03;
                params.IntensityThr = .02;
                
                params.MS.sptSig = 3;    %to find bifurcations use a smaller spt. kernel
                params.MS.colSig = .5;    %to find bifurcations use a smaller spt. kernel
                params.varSig = true;
                
                % for 17 => 7; sum(cv) = 30 => 2.5;  .45*sum(cv)+14.65
                % for 17 => 5; sum(cv) = 30 => 2.5;  .25*sum(cv)+9.25
                params.varSigMax = 7;
                params.varSigMin = 2.5;
                params.varSigMinBound = 2.5;

                x=[17 27];
                y=[params.varSigMax params.varSigMin];
                a=(y(2)-y(1))/(x(2)-x(1));
                b=y(1)-a*x(1);
                params.varSigCoef = [a,b];
                
                params.smoothImages = true;
                h = fspecial('gaussian',[3 3],.5);
                
                params.forePerc = 50;
                params.filt = false;

            end
            
        case 6
            for i=1
                seedxyz = [15.074,412.01,10];
                seed = [seedxyz(3) seedxyz(2) seedxyz(1)];
                seed = seed + [1 0 0];
                params.dim.z = 101;
                
                params.tangVecprev = [1;-1;1];   %initial direction of pc
                params.sptSig = 5;
                params.colSig = .5;
                params.kernelWeightFactor = .2; %power of the weight term in tracing step
                params.Factors.neig=2;  % weights of vectorial tracing
                params.Factors.vprev=1;
                params.Tracing.lenTang=1;    %tangent vec
                params.Tracing.lenVal=1;  %color
                params.branchFibStart = 4;
                params.Bif.varThr = false;
                params.Bif.remLowThr = .5;
                params.Bif.Rmin = 4;
                params.Bif.Rmax = 14;
                params.Bif.clusterSize = 3;
                params.Bif.ClusterDistThr = 5;
                params.Bif.centroidImtensityThr = .3;
                params.Bif.intensityThr = .05;
                params.remIntensityThr = .15;
                params.IntensityThr = .1;
                
                params.MS.sptSig = 3;    %to find bifurcations use a smaller spt. kernel
                params.MS.colSig = .5;    %to find bifurcations use a smaller spt. kernel
                params.varSig = true;
                
                % for 17 => 7; sum(cv) = 27 => 2.5;  .45*sum(cv)+14.65
                % for 17 => 5; sum(cv) = 27 => 2.5;  .25*sum(cv)+9.25
                % ax+b=y;
                params.varSigMax = 7;
                params.varSigMin = 2.5;
                params.varSigMinBound = 1.5;
                x=[17 27];
                y=[params.varSigMax params.varSigMin];
                a=(y(2)-y(1))/(x(2)-x(1));
                b=y(1)-a*x(1);
                params.varSigCoef = [a,b];
                
                params.smoothImages = false;
                
                params.forePerc = 100;
                params.filt = true;

                
            end
    end
    params.seed = seed;
    params.directory = directory;
    % Load Images
    IM=zeros(params.dim.z,params.dim.y,params.dim.x);
    % IM2=zeros(round(params.dim.z*params.zdist),params.dim.y,params.dim.x);
    IM2=zeros(round(params.dim.z*1),params.dim.y,params.dim.x);
    dim = size(IM2);
    for i=1:params.dim.z
        loadName = sprintf('%sOP_%d/%02d.tif',params.directory,params.dataset,i);
        imtemp = im2double(imread(loadName));
        if params.smoothImages
            %             imtemp = imfilter(imtemp,h);
            imtemp = medfilt2(imtemp);
        end
        IM(i,:,:) = imresize(imtemp,1);
    end
    for i=1:params.dim.y
        IM2(:,i,:) = imresize(squeeze(IM(:,i,:)),[dim(1) dim(3)]);
    end
    Im = IM2;
    data = Im;
    clearvars -except Im dim params data
    clearvars -except data datamed dim params
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    [Y X Z]= meshgrid(1:dim(2),1:dim(1),1:dim(3));
    params.zdist = 1;
    indVec = [round(params.zdist*X(:)),Y(:),Z(:)];
    dataVec = [indVec data(:)];
    
    foreIm = logical(data(:)>(max(data(:))/params.forePerc)); %intensity threshold
    
    imVec = dataVec(foreIm,:);
    p = imVec(:,1:3);
    c = imVec(:,4);
    f = [p c];
    clearvars -except f foreIm dim data params dataVec
    %%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Create KNN graph
    ptrtree=BuildGLTree(f(:,1:3));
    k=500;  %500->R~5, 250->R~4
    [KNNG,distances]=KNNSearch(f(:,1:3),f(:,1:3),ptrtree,k);
    N = length(KNNG);
    indforeIm = find(foreIm);
    IndI = zeros(1,length(foreIm));
    IndI(indforeIm) = 1:N;
    ForImage = reshape(foreIm,dim(1),dim(2),dim(3));
    %     save(['OlFactory_vs_scaled_knn',num2str(k),'.mat'], '-V7.3')
    %%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    f(:,4) = f(:,4)/max(f(:,4));
    fori = f;
    f_filt = f;
    parfor ind=1:N
        if ~mod(ind,1000)
            ind
        end
        indknn = KNNG(ind,1:k);
        f_filt(ind,end) = max(f(indknn,end));
    end
    params.f_filt=f_filt;
    
    IT =  zeros(dim(1),dim(2),dim(3));
    IT(find(IndI)) = f_filt(:,end);
    figure, imshow(squeeze(IT(29,:,:)))
    %%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % vocal covariances
    parfor i=1:length(f)
        cv=(diag(cov(f(KNNG(i,:),1:3))));
        varSptSig(i)=params.varSigCoef(1)*sum(cv)+params.varSigCoef(2); %sum(cv) = 17 => 7; sum(cv) = 30 => 2.5;
    end
    % k=500=> R~7, siunce sum([1:2:11].^2+13^2)=455~=500
    % from training:
    % sum(cv) = 17 => 5; sum(cv) = 30 => 2.5;
    varSptSig(varSptSig>params.varSigMax)=params.varSigMax;
    varSptSig(varSptSig<params.varSigMinBound)=params.varSigMinBound;
    params.varSptSig=varSptSig;
    
    %%
    f = fori;
    % indexOfSeed=find(all(f(:,1:3)==repmat(round(params.seed.*[params.zdist 1 1]),length(f),1),2));
    %     indexOfSeed=find(all(f(:,1:3)==repmat(round(params.seed.*[params.zdist 1 1]),length(f),1),2));
    %     I=zeros(3,length(KNNG));
    Inext=zeros(3,length(KNNG));
    indknn_ = 1:k;
    
    params.featDimP = 3;
    params.featDimC = 1;
    params.featDim = params.featDimC + params.featDimP;
    
    params.ani = 0;
    params.invCi = [];
    params.invCxi=[];
    params.knnNum = k;
    params.fullH = 0;   %0-> uses only Hpp, 1: full H
    
    %     params.sptSigx = params.sptSig;
    %     params.sptSigy = params.sptSig;
    %     params.sptSigz = params.sptSig;
    params.scale = 1;
    params.normalUpdateThr = .5;
    
    
    if params.varSig
        varSpt=params.varSptSig;
        % Calculate initial PC projections using variable size kernels
        projPoints2 = zeros(params.featDimP,N);
        projPointsf2 = zeros(params.featDimP,N);
        projVecs2 = zeros(params.featDimP,params.featDimP,N);
        Covx2 = zeros(3,3,N);
        tic
        parfor indexOfVoxel=1:N
            sptSig = varSpt(indexOfVoxel)*1;
            if ~mod(indexOfVoxel,1000)
                indexOfVoxel
            end
            indknn = KNNG(indexOfVoxel,1:k);
            f_i = f(indknn,:)';
            x = f_i(:,end);
            xg = round(x(1:3)./[params.zdist;1;1]);
            
            [pcUpdate_normal subSpace pctang SI vals] = pcProj(x,f_i,params,f_i(4,:).^1,sptSig);
            proj_normalD = pcUpdate_normal;%./[params.zdist;1;1];
            it = 1;
            ind_it=1;
            
            if any(abs(proj_normalD) >params.normalUpdateThr)
                continue;
            else
                %             while any(abs(pcUpdate_normal./[params.zdist;1;1])>.05) &  ind_it & it<=100
                while any(abs(pcUpdate_normal)>.05) &  ind_it & it<=100
                    [pcUpdate_normal subSpace pctang SI vals] = pcProj(x,f_i,params,f_i(4,:).^1,sptSig);
                    proj_normalD = proj_normalD + pcUpdate_normal./[params.zdist;1;1];
                    x(1:params.featDimP) = x(1:params.featDimP) + pcUpdate_normal;
                    %                 hold on, plot3(x(3),x(2),x(1),'g.')
                    xg = round(x(1:3)./[params.zdist;1;1]);
                    xg = max([1;1;1],xg);
                    xg = min(dim(:),xg);
                    ind_it = IndI(sub2ind(dim(1:3),xg(1,:),xg(2,:),xg(3,:)));
                    if ~ind_it %| any(abs(pcUpdate_normal./[params.zdist;1;1]) >params.normalUpdateThr)
                        continue;
                    end
                    x(params.featDimP+1:end) = f(ind_it,(params.featDimP+1:end));
                    
                    indknn = KNNG(ind_it,1:k);
                    f_i = f(indknn,:)';
                    it = it+1;
                end
                
                if it>100 | ~ind_it
                    continue;
                end
                
                projPoints2(:,indexOfVoxel) = xg;
                projPointsf2(:,indexOfVoxel) = x(1:params.featDimP);
                projVecs2(:,:,indexOfVoxel) = [pctang(:) subSpace];
                temp = inv(SI);
                Covx2(:,:,indexOfVoxel) = sign(temp(1))*temp;
            end
        end
        toc
    else
        %%
        % Calculate initial PC projections
        projPoints1 = zeros(params.featDimP,N);
        projPointsf1 = zeros(params.featDimP,N);
        projVecs1 = zeros(params.featDimP,params.featDimP,N);
        Covx1 = zeros(3,3,N);
        %     matlabpool(4)
        tic
        parfor indexOfVoxel=1:N
            sptSig = params.sptSig;
            if ~mod(indexOfVoxel,1000)
                indexOfVoxel
            end
            indknn = KNNG(indexOfVoxel,1:k);
            f_i = f(indknn,:)';
            x = f_i(:,end);
            xg = round(x(1:3)./[params.zdist;1;1]);
            
            [pcUpdate_normal subSpace pctang SI vals] = pcProj(x,f_i,params,f_i(4,:).^1,sptSig);
            proj_normalD = pcUpdate_normal;%./[params.zdist;1;1];
            it = 1;
            ind_it=1;
            
            if any(abs(proj_normalD) >params.normalUpdateThr)
                continue;
            else
                %             while any(abs(pcUpdate_normal./[params.zdist;1;1])>.05) &  ind_it & it<=100
                while any(abs(pcUpdate_normal)>.05) &  ind_it & it<=100
                    [pcUpdate_normal subSpace pctang SI vals] = pcProj(x,f_i,params,f_i(4,:).^1,sptSig);
                    proj_normalD = proj_normalD + pcUpdate_normal./[params.zdist;1;1];
                    x(1:params.featDimP) = x(1:params.featDimP) + pcUpdate_normal;
                    %                 hold on, plot3(x(3),x(2),x(1),'g.')
                    xg = round(x(1:3)./[params.zdist;1;1]);
                    xg = max([1;1;1],xg);
                    xg = min(dim(:),xg);
                    ind_it = IndI(sub2ind(dim(1:3),xg(1,:),xg(2,:),xg(3,:)));
                    if ~ind_it %| any(abs(pcUpdate_normal./[params.zdist;1;1]) >params.normalUpdateThr)
                        continue;
                    end
                    x(params.featDimP+1:end) = f(ind_it,(params.featDimP+1:end));
                    
                    indknn = KNNG(ind_it,1:k);
                    f_i = f(indknn,:)';
                    it = it+1;
                end
                
                if it>100 | ~ind_it
                    continue;
                end
                
                projPoints1(:,indexOfVoxel) = xg;
                projPointsf1(:,indexOfVoxel) = x(1:params.featDimP);
                projVecs1(:,:,indexOfVoxel) = [pctang(:) subSpace];
                temp = inv(SI);
                Covx1(:,:,indexOfVoxel) = sign(temp(1))*temp;
            end
        end
        toc
    end
    % save vs7_PC_knn300_.mat
    %%
    Im=data/max(data(:));
    Im2(:,:,:,1)=Im;
    Im2(:,:,:,2)=Im;
    Im2(:,:,:,3)=Im;
    
    %%
    if 1
        if ~params.varSig
            indPC = find(any(projPointsf1));
            projPointsSet = projPointsf1(:,indPC);
            figure,
            imshow(squeeze(Im2(15,:,:,:)),[]),hold on,
            plot3(projPointsSet(3,:),projPointsSet(2,:),projPointsSet(1,:),'g.')
            title([num2str(params.PCcol),';',num2str(params.PCspt),';',num2str(params.normalUpdateThr)])
            projPoints = projPoints1;
            projPointsf = projPointsf1;
            projVecs = projVecs1;
            Covx = Covx1;
        else
            indPC = find(any(projPointsf2));
            projPointsSet = projPointsf2(:,indPC);
            figure,
            imshow(squeeze(Im2(14,:,:,:)),[]),hold on,
            plot3(projPointsSet(3,:),projPointsSet(2,:),projPointsSet(1,:),'g.')
            title([num2str(params.colSig),';',num2str(params.sptSig),';',num2str(params.normalUpdateThr)])
            projPoints = projPoints2;
            projPointsf = projPointsf2;
            projVecs = projVecs2;
            Covx = Covx2;
        end
    end
%     save(['OlFactory_0',num2str(params.dataset),'_preprocess'])
end
%%  Trace a fiber

Im=data/max(data(:));
Im2(:,:,:,1)=Im;
Im2(:,:,:,2)=Im;
Im2(:,:,:,3)=Im;
bifCheck = 0;

tangVecprev = params.tangVecprev;
indexOfSeed=find(all(f(:,1:3)==repmat(round(params.seed.*[params.zdist 1 1]),length(f),1),2));
if isempty(indexOfSeed)
    indexOfSeed=find(all(f(:,1:3)==repmat(floor(params.seed.*[params.zdist 1 1]),length(f),1),2));
end
params.visual = 0;
numOfFiber=1;
%%
initSeed = indexOfSeed;
params.initSeed = indexOfSeed;
PC=[];
PCAll = [];
params.fig=31;
params.visual = 0;
% [PC_ numOfFiber_ PCAll_ projPointsfRem_]=deal(PC,numOfFiber, PCAll, projPointsfRem);
%%
params.remCheck = 0;
params.permute=0;
% params.filt = true;
[PC1 numOfFiber1 PCAll1 projPointsRem1 projPointsfRem1] = branchIt(f,indexOfSeed,tangVecprev,Im2,KNNG,IndI,PC,numOfFiber,initSeed,projPoints,projPointsf,projVecs,PCAll,params);
[PCb,numOfFiberb,PCAllb,projPointsRemb,projPointsfRemb] = deal(PC1,numOfFiber1,PCAll1,projPointsRem1,projPointsfRem1);
%%
[PC1,numOfFiber1,PCAll1,projPointsRem1,projPointsfRem1] = deal(PCb,numOfFiberb,PCAllb,projPointsRemb,projPointsfRemb);
% check any remaning points
params.remCheck = 1;
endTrace = 0;
while ~endTrace
    [PC1 numOfFiber1 PCAll1 projPointsRem1 projPointsfRem1 endTrace] = checkRem(f,Im2,KNNG,IndI,PC1,numOfFiber1,[],projPointsRem1,projPointsfRem1,projVecs,PCAll1,params);
end
%%
% params.remCheck = 0;
% params.permute=1;
% [PC2 numOfFiber2 PCAll2 projPointsfRem2] = branchIt(f,indexOfSeed,tangVecprev,Im2,KNNG,IndI,PC,numOfFiber,initSeed,projPoints,projPointsf,projVecs,PCAll,params);
% 
% % check any remaning points
% params.remCheck = 1;
% endTrace = 0;
% while ~endTrace
%     [PC2 numOfFiber2 PCAll2 projPointsfRem2 endTrace] = checkRem(f,Im2,KNNG,IndI,PC2,numOfFiber2,[],projPoints,projPointsfRem2,projVecs,PCAll2,params);
% end
%%

%%
% create connectivity matrix
% PC= PCson;
folder = directory;%'C:/ERHAN/WORK/DATA/DIADEM/Olfactory Projection Fibers v2/Qualifier/';
% PC1=PC
[SWC1 S1 B1 wS1] = createConMatrix(PC1,f,params);
file_1 = fopen([folder,'Qual_OF_Test_',num2str(params.dataset),'vs_S1.swc'],'w');
fprintf(file_1,'# Neurolucida to SWC conversion from L-Measure.\n');
fprintf(file_1,'%d %d %4.3f %4.3f %4.3f %d %d\n',SWC1');
fclose(file_1);
isconnected(S1)
%%

% [SWC2 S2 B2 wS2] = createConMatrix(PC2,f,params);
% file_1 = fopen([folder,'Qual_OF_Test_',num2str(params.dataset),'vs_S2.swc'],'w');
% fprintf(file_1,'# Neurolucida to SWC conversion from L-Measure.\n');
% fprintf(file_1,'%d %d %4.3f %4.3f %4.3f %d %d\n',SWC2');
% fclose(file_1);
%%

% S = (S1+S1)/2;
% B=[B1;B2];
% %%
% clc
% isconnected(S)
% isloop(S)

% [D,pred,cl] = graphtraverse(S,B(1,2),'METHOD','BFS');

