function [centers eliminateProj vecs]= bifurcationDetection(projPointsf,Y,indPC,params,Rtrace,projVecs)

X = projPointsf(:,indPC)';
Xvec = squeeze(projVecs(:,1,indPC))';

pD = (pdist2(X,Y));
pD = min(pD,[],2);

eliminateProj = indPC(pD<params.Bif.Rmin);
vecs=[];

if params.Bif.varThr
    bif = pD<params.Bif.Rmax &pD>Rtrace(:);
else
    bif = pD<params.Bif.Rmax &pD>params.Bif.Rmin;
end
indbf = find(any(bif,2));
dataBfP = X(indbf,:);
dataVecBfP = Xvec(indbf,:);

paramsMS = params;
paramsMS.sptSig = params.MS.sptSig;    %to find bifurcations use a smaller spt. kernel
paramsMS.colSig = params.MS.colSig;    %to find bifurcations use a smaller spt. kernel
paramsMS.featDimC=0;
paramsMS.featDimP=3;
paramsMS.featDim=paramsMS.featDimC+paramsMS.featDimP;

if(0)
    % Apply mean shift algorithm
    [~, ~, ~, ~, centers] = meanShift(dataBfP',paramsMS);
else
    % Apply medioidshift with ISOMAP
    %     D  = dist(dataBfP').^2; % Compute Distance Matrix
    %     [ar_mode2, iter] = medoidshift(D,5); % Complete Medoidshift Algorithm
    %     visualizeClustering(ar_mode2,dataBfP'); % Visualize Result
    %     title(sprintf('Medoidshift Result: %d Iterations',iter));
    D  = dist(dataBfP').^2; % Compute Distance Matrix
    options.dims = 1:10;
    options.verbose = 0;
    if isempty(D)
        centers=[];
        return;
    end
    [D_new] = IsomapIID(D, 'k', size(D,1), options); %http://isomap.stanford.edu
    [ar_mode2, iter, Dclust, S] = medoidshift(D_new,paramsMS.sptSig,dataBfP'); % Complete Medoidshift Algorithm
    B = unique(ar_mode2);
    % if returned weighted pairwise mode distances are very small, merge them
    Dm = triu(Dclust<params.Bif.ClusterDistThr);
    Dm(1:length(Dclust)+1:end) = 0;
    [a b] = find(Dm);
    for i=1:size(a,1)
        ar_mode2(ar_mode2 == B(a(i))) = B(b(i));
    end
    B = unique(ar_mode2);
    
    c = zeros(1,length(B));
    V = zeros(3,length(B));
    for i=1:length(B)
        c(i)=sum(ar_mode2==B(i));
        if nargout>2 & c(i)>params.Bif.clusterSize
            if params.remCheck
            temp=dataBfP(ar_mode2==B(i),:);
            temp = temp-repmat(mean(temp,1),c(i),1);
            [v,~]=eigs(temp'*temp,1);
            V(:,i) = v;
            else
                temp=dataVecBfP(ar_mode2==B(i),:);
                temp=repmat(sign(temp*temp(1,:)'),1,3).*temp;
                V(:,i) = normr(median(temp));
            end
        end
    end
    %     c/sum(c)>
    centers = dataBfP(B(c>params.Bif.clusterSize),:)';
    if nargout>2
        vecs = V(:,(c>params.Bif.clusterSize));
    end
        
    if params.visual
        visualizeClustering(ar_mode2,dataBfP'); % Visualize Result
        title(sprintf('Medoidshift Result: %d Iterations',iter));
    end
end
%%
if params.visual
    hold on,plot3(Y(:,3),Y(:,2),Y(:,1),'r.')
    xlim([0 520])
    ylim([0 520])
    zlim([0 100])
end

